package	D2P::Generator::PHPClasses;
use base 'D2P::Generator';
use D2P::Parser;

sub	strToPgId
{
	my	$string	= shift;

	return $string if $string =~ /^\".*\"$/;

	return "\"$string\"" if (length $string < 32);

	$string =~ s/\_+/_/g;
	return "\"$string\"" if (length $string < 32);

	my	@list	= split /_/, $string;
	my	$old	= $string;
	my	$lsz	= @list;
	my	$rest	= 32 - @list;
	my	$max;

	$string = "";
	foreach my $it (@list)
	{
		$max = $rest / $lsz;
		$max =~ s/\.(.*)$//;
		$max += 1 if ($1 && $1 > 5 && $max < $rest);
		$string .= '_' if $string;

		if	(length $it > $max)
		{
			$string .= substr $it, 0, $max;
			$rest -= $max;
		}
		else
		{
			$string .= $it;
			$rest -= length $string;
		}

		$lsz--;
	}

	print STDERR "WARNING: name \"$old\" changed to \"$string\"\n";
	return "\"$string\"";
}


###
## Strip double quotes from a Postgres name
#
sub	stripDQuotes
{
	my $n = shift;
	$n =~ s/\"//g;
	return	$n;
}


sub	init
{
	my	$this	= shift;

	$this->SUPER::init();
	$this->{config}->{phpout} = "";
}


my $phpout = "";

sub	commandLine
{
	return	(
		"-p|--php"	=> {
			desc	=> "Output PHP mirror classes to the specified file.",
			uniq	=> 1,
			args	=> ["phpout"]
		}
	);
}


###
## Create complete PHP output
#
sub	generate
{
	my	$this = shift;
	$phpout = $this->{config}->{phpout};

	return unless $phpout;
	unless	(open(PHPOUT, ">$phpout"))
	{
		print STDERR "Warning : could not create PHP output file $phpout\n";
		return;
	}
	&createPHPHeader();
	&createPHPEnums();
	&createPHPClasses();
	&createPHPFooter();
	close PHPOUT;
}


###
## Create PHP header
#
sub	createPHPHeader
{
	$DEBUG && print STDERR " Writing header\n";

	print PHPOUT "<?php\n\n/*\n";

	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'box' && $g->{topLevelText} ne '' && @{$g->{classes}} == 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\n$//;

		my	@lines = split /\n/, $title;
		foreach my $l (@lines)
			{ print PHPOUT "$l\n"; }
		print PHPOUT "\n";
		last;
	}

	print PHPOUT "Input file  : $infile\n";
	print PHPOUT "\n";
	print PHPOUT "Defines PHP classes for the following modules : \n";
	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'box' && $g->{topLevelText} ne '' && @{$g->{classes}} != 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\s+/ /g;

		print PHPOUT " - $title\n";
	}

	my	@requiredModules = ();
	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'pack' && $g->{topLevelText} ne '' && @{$g->{classes}} != 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\s+/ /g;
		push @requiredModules, $title;
	}
	if	(@requiredModules)
	{
		print PHPOUT "\n";
		print PHPOUT "Requires the following modules :\n - ";
		print PHPOUT join ("\n - ", @requiredModules);
		print PHPOUT "\n";
	}
	print PHPOUT "*/\n\nrequire_once(\"d2p_manager.inc\");\n";
	print PHPOUT "\n\n";
}


###
## Create PHP classes for enumerations according to data created by checkAndModifyInput()
#
sub	createPHPEnums
{
	foreach my $re (@enumerations)
	{
		my	$enum = {%$re};
		$enum->{tablename} =~ s/\"//g;
		$tn = $enum->{tablename};
		print PHPOUT "\n// Enumeration for member $enum->{fieldname}\n\n";
		print PHPOUT "\$d2p_cl_$tn = \&\$d2pManager->newTable(NULL, ";
		print PHPOUT "\"$tn\", \"$tn\");\n";
		print PHPOUT "\$d2p_cl_$tn\->addField(\"id\", \"SERIAL\", true);\n";
		print PHPOUT "\$d2p_cl_$tn\->addField(\"txt\", \"TEXT\", true);\n";
		print PHPOUT "\$d2p_cl_$tn\->setIdentifier(\"id\");\n";
		print PHPOUT "\$d2p_cl_$tn\->setDescription('";
		print PHPOUT '$d = $this->fields["txt"]["current_value"];' . "');\n";
	}
}


###
## Create PHP classes for each table
#
sub	createPHPClasses
{
	foreach my $gro (sort keys %groupsOrdered)
	{
		foreach my $g (@{$groupsOrdered{$gro}})
		{
			next unless ($g->{type} eq 'box' && @{$g->{classes}});

			my	$title = $g->{topLevelText};
			$title =~ s/\n/\n\/\/ /g;
			print PHPOUT "\n\n\n// Tables for module $title\n\n";
			foreach my $o (@{$g->{classes}})
			{
				my	$x = 0;
				my	@pkey = ();
				my	@codeMethodes = ();
				my	@codeDescriptions = ();
				my	@codeChamps = ();
				my	@codeFormulaire = ();
				my	$tnam = &strToPgId($o->{name});
				my	$on = $o->{name};

				print PHPOUT "\$d2p_cl_$on = \&\$d2pManager->newTable(";
				if	(@{$o->{inherits}})
				{
					print PHPOUT "\"" . $o->{inherits}->[0]->{name} . "\"";
				}
				else
				{
					print PHPOUT "NULL";
				}
				print PHPOUT ", \"$o->{name}\", $tnam);\n";
				foreach my $f (@{$o->{fields}})
				{
					my	$fnam = &strToPgId($f->{name});
					print PHPOUT "\$d2p_cl_$on\->addField($fnam, \"";
					print PHPOUT "$f->{type}\", ". ($f->{not_null} ? "true" : "false");
					if	($f->{references})
					{
						my	($table, $field) = map {
									&strToPgId($_);
								} (split (/\./, $f->{references}));
						print PHPOUT ", $table";
					}
					print PHPOUT ");\n";
				}
				my	@opk = split /,/, $o->{pkey};
				@opk = map { &strToPgId($_) } (@opk);
				print PHPOUT "\$d2p_cl_$on\->setIdentifier(";
				if	(@opk > 1)
				{
					print PHPOUT "array(" . join(", ", @opk) . ")";
				}
				else
				{
					print PHPOUT $opk[0];
				}
				print PHPOUT ");\n\n";
			}
		}
	}
}


###
## Create PHP footer
#
sub	createPHPFooter
{
	$DEBUG && print STDERR " Writing footer\n";
	print PHPOUT "\n\n// PHP code generated by dia2postgres.pl\n";
	print PHPOUT "// $VERSION\n"; 
	print PHPOUT "// on " . `date`;
	print PHPOUT "\n?>\n";
}




1;
