package	D2P::Generator::PgDrop;
use base 'D2P::Generator';
use D2P::Parser;


sub	strToPgId
{
	my	$string	= shift;

	return $string if $string =~ /^\".*\"$/;

	return "\"$string\"" if (length $string < 32);

	$string =~ s/\_+/_/g;
	return "\"$string\"" if (length $string < 32);

	my	@list	= split /_/, $string;
	my	$old	= $string;
	my	$lsz	= @list;
	my	$rest	= 32 - @list;
	my	$max;

	$string = "";
	foreach my $it (@list)
	{
		$max = $rest / $lsz;
		$max =~ s/\.(.*)$//;
		$max += 1 if ($1 && $1 > 5 && $max < $rest);
		$string .= '_' if $string;

		if	(length $it > $max)
		{
			$string .= substr $it, 0, $max;
			$rest -= $max;
		}
		else
		{
			$string .= $it;
			$rest -= length $string;
		}

		$lsz--;
	}

	print DROPOUT STDERR "WARNING: name \"$old\" changed to \"$string\"\n";
	return "\"$string\"";
}


sub	init
{
	my	$this	= shift;

	$this->SUPER::init();
	$this->{config}->{outfile} = "";
}


sub	commandLine
{
	my	$this	= shift;

	return	(
		"-d|--drop"	=> {
			desc	=> "Output a PostgreSQL script meant to destroy the database "
				. "to the specified file.",
			args	=> ["outfile"],
			uniq	=> 1
		}
	);
}


my $outfile = "";


sub	generate
{
	my	$this = shift;

	$outfile = $this->{config}->{outfile};
	return unless $outfile;

	## Prepare output
	open(DROPOUT, ">$outfile");

	&createSQLHeader();
	&dropSQLTables();
	&dropSQLEnums();
	&createSQLFooter();

	close DROPOUT;
}


###
## Create SQL header
#
sub	createSQLHeader
{
	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'box' && $g->{topLevelText} ne '' && @{$g->{classes}} == 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\n$//;

		my	@lines = split /\n/, $title;
		foreach my $l (@lines)
			{ print DROPOUT "-- $l\n"; }
		print DROPOUT "\n";
		last;
	}

	print DROPOUT "\n-- \n-- DROP SCRIPT\n-- \n\n";
	print DROPOUT "-- Will drop the following modules : \n";
	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'box' && $g->{topLevelText} ne '' && @{$g->{classes}} != 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\s+/ /g;

		print DROPOUT "--  $title\n";
	}
	print DROPOUT "\n";
	print DROPOUT "BEGIN TRANSACTION;\n\n\n";
}


###
## Drop SQL tables for each included module
#
sub	dropSQLTables
{
	foreach my $gro (sort keys %groupsOrdered)
	{
		foreach my $g (@{$groupsOrdered{$gro}})
		{
			next unless ($g->{type} eq 'box' && @{$g->{classes}});

			my	$title = $g->{topLevelText};
			$title =~ s/\n/\n-- /g;
			print DROPOUT "-- Tables for $title\n\n";
			foreach my $o (@{$g->{classes}})
			{
				my	$x = 0;
				my	@pkey = ();
				my	@references = ();
				my	$tnam = strToPgId($o->{name});
				(my	$tnVal = $tnam) =~ s/\"//g;

				print DROPOUT "DELETE FROM d2p_ref_list WHERE src_table = '$tnVal' OR dest_table = '$tnVal';\n";
				print DROPOUT "DELETE FROM d2p_inh_cache WHERE child = '$tnVal';\n";
				print DROPOUT "DELETE FROM d2p_lst_list WHERE d2p_name = '$tnVal';\n";
				print DROPOUT "DROP TABLE $tnam;\n";
				print DROPOUT "\n";
			}
			print DROPOUT "\n";
		}
	}
}


###
## Drop SQL tables for enumerations according to data created by checkAndModifyInput()
#
sub	dropSQLEnums
{
	foreach my $enum (@enumerations)
	{
		my	$l = 0;
		(my	$tnVal = $enum->{tablename}) =~ s/\"//g;
		foreach my $w1 (@{$enum->{contents}})
		{
			my	$w1l = length($w1);
			($w1l > $l) && ($l = $w1l);
		}

		print DROPOUT "-- Enumeration table for " . $enum->{fieldname} . "\n\n";
		print DROPOUT "DELETE FROM d2p_ref_list WHERE dest_table = '$tnVal';\n";
		print DROPOUT "DELETE FROM d2p_lst_list WHERE d2p_name = '$tnVal';\n";
		print DROPOUT "DROP TABLE $enum->{tablename};\n";
		print DROPOUT "\n";
	}
}


###
## Create SQL footer
#
sub	createSQLFooter
{
	print DROPOUT "\n-- \n\n";
	print DROPOUT "SELECT f_d2p_inh_update();\nSELECT f_d2p_ref_update();\n\n";
	print DROPOUT "COMMIT TRANSACTION;\n\n";
	print DROPOUT "-- SQL drop script generated by dia2postgres.pl\n";
	print DROPOUT "-- $VERSION\n"; 
	print DROPOUT "-- on " . `date`;
}



1;
